#!/usr/bin/env python3

import argparse

from migen import *

from litex.boards.platforms import nexys_video

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import mem_decoder
from litex.soc.integration.soc_sdram import *
from litex.soc.integration.builder import *

from litedram.modules import MT41K256M16
from litedram.phy import s7ddrphy

from liteeth.phy.s7rgmii import LiteEthPHYRGMII
from liteeth.core.mac import LiteEthMAC

# CRG ----------------------------------------------------------------------------------------------

class _CRG(Module):
    def __init__(self, platform, sys_clk_freq):
        self.clock_domains.cd_sys = ClockDomain()
        self.clock_domains.cd_sys4x = ClockDomain(reset_less=True)
        self.clock_domains.cd_sys4x_dqs = ClockDomain(reset_less=True)
        self.clock_domains.cd_clk200 = ClockDomain()
        self.clock_domains.cd_clk100 = ClockDomain()

        self.submodules.pll = pll = S7MMCM(speedgrade=-1)
        self.comb += pll.reset.eq(~platform.request("cpu_reset"))
        pll.register_clkin(platform.request("clk100"), 100e6)
        pll.create_clkout(self.cd_sys, sys_clk_freq)
        pll.create_clkout(self.cd_sys4x, 4*sys_clk_freq)
        pll.create_clkout(self.cd_sys4x_dqs, 4*sys_clk_freq, phase=90)
        pll.create_clkout(self.cd_clk200, 200e6)
        pll.create_clkout(self.cd_clk100, 100e6)

        self.submodules.idelayctrl = S7IDELAYCTRL(self.cd_clk200)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCSDRAM):
    csr_map = {
        "ddrphy": 16,
    }
    csr_map.update(SoCSDRAM.csr_map)
    def __init__(self, **kwargs):
        platform = nexys_video.Platform()
        sys_clk_freq = int(100e6)
        SoCSDRAM.__init__(self, platform, clk_freq=sys_clk_freq,
                         integrated_rom_size=0x8000,
                         integrated_sram_size=0x8000,
                         **kwargs)

        self.submodules.crg = _CRG(platform, sys_clk_freq)

        # sdram
        self.submodules.ddrphy = s7ddrphy.A7DDRPHY(platform.request("ddram"), sys_clk_freq=sys_clk_freq)
        sdram_module = MT41K256M16(sys_clk_freq, "1:4")
        self.register_sdram(self.ddrphy,
                            sdram_module.geom_settings,
                            sdram_module.timing_settings)

# EthernetSoC --------------------------------------------------------------------------------------

class EthernetSoC(BaseSoC):
    csr_map = {
        "ethphy": 18,
        "ethmac": 19
    }
    csr_map.update(BaseSoC.csr_map)

    interrupt_map = {
        "ethmac": 3,
    }
    interrupt_map.update(BaseSoC.interrupt_map)

    mem_map = {
        "ethmac": 0x30000000,  # (shadow @0xb0000000)
    }
    mem_map.update(BaseSoC.mem_map)

    def __init__(self, **kwargs):
        BaseSoC.__init__(self, **kwargs)

        self.submodules.ethphy = LiteEthPHYRGMII(self.platform.request("eth_clocks"),
                                                 self.platform.request("eth"))
        self.submodules.ethmac = LiteEthMAC(phy=self.ethphy, dw=32,
            interface="wishbone", endianness=self.cpu.endianness)
        self.add_wb_slave(mem_decoder(self.mem_map["ethmac"]), self.ethmac.bus)
        self.add_memory_region("ethmac", self.mem_map["ethmac"] | self.shadow_base, 0x2000)

        self.crg.cd_sys.clk.attr.add("keep")
        self.ethphy.crg.cd_eth_rx.clk.attr.add("keep")
        self.ethphy.crg.cd_eth_tx.clk.attr.add("keep")
        self.platform.add_period_constraint(self.crg.cd_sys.clk, 10.0)
        self.platform.add_period_constraint(self.ethphy.crg.cd_eth_rx.clk, 8.0)
        self.platform.add_period_constraint(self.ethphy.crg.cd_eth_tx.clk, 8.0)
        self.platform.add_false_path_constraints(
            self.crg.cd_sys.clk,
            self.ethphy.crg.cd_eth_rx.clk,
            self.ethphy.crg.cd_eth_tx.clk)

# Build --------------------------------------------------------------------------------------------

def main():
    parser = argparse.ArgumentParser(description="LiteX SoC on Nexys Video")
    builder_args(parser)
    soc_sdram_args(parser)
    parser.add_argument("--with-ethernet", action="store_true",
                        help="enable Ethernet support")
    args = parser.parse_args()

    cls = EthernetSoC if args.with_ethernet else BaseSoC
    soc = cls(**soc_sdram_argdict(args))
    builder = Builder(soc, **builder_argdict(args))
    builder.build()


if __name__ == "__main__":
    main()



import obt.command 

IOS_VERSION = "iphoneos15.2" # latest supported by both bigsur and monterey/x86_64 and M1

_macsdkstr = None
_iossdkstr = None



def macsdkstr():
  global _macsdkstr
  if _macsdkstr==None:
    _macsdkstr = obt.command.capture([
     "xcodebuild",
     "-version",
     "-sdk", "macosx",
     "Path"],do_log=False).splitlines()
  return _macsdkstr

def iossdkstr():
  global _iossdkstr
  if _iossdkstr==None:
    _iossdkstr = obt.command.capture([
     "xcodebuild",
     "-version",
     "-sdk", IOS_VERSION,
     "Path"],do_log=False).splitlines()
  return _iossdkstr



import os, tarfile
from obt import dep, host, path, make, pathtools, log
from obt.deco import Deco
from obt.wget import wget
from obt import command, patch, dep

VER = "X"
NAME = "pytorch"

class _pytorch_from_source(dep.StdProvider):

  def __init__(self): ############################################
    super().__init__(NAME,NAME)
    self._builder = self.createBuilder(dep.CustomBuilder)
    self.VERSION = VER
    cmd_list = ["pip3","install","--pre",
                "torch","torchvision","torchaudio",
                "--index-url",
                "https://download.pytorch.org/whl/nightly/cu128"]
    self._builder._cleanbuildcommands += [command.Command(cmd_list)]
  ########################################################################
  @property
  def _fetcher(self):
    return dep.NopFetcher(name=NAME)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return True
  ########################################################################
  def areRequiredBinaryFilesPresent(self):
    return (path.bin()/"pkg-config").exists()
###############################################################################
class _pytorch_from_homebrew(dep.HomebrewProvider):
  def __init__(self):
    super().__init__(NAME,NAME)
    self.VERSION = "homebrew"
###############################################################################
class _pytorch_for_mps(dep.StdProvider):
  def __init__(self):
    super().__init__(NAME,NAME)
    self.VERSION = "MPS"
    self._builder = self.createBuilder(dep.CustomBuilder)
    #################
    # create patch file
    #################
    DIFF = """\
--- serialization.py
+++ serialization.py
@@ -2077,1 +2077,2 @@
-            nbytes = numel * torch._utils._element_size(dtype)
+            nbytes = numel * torch._utils._element_size(dtype)
+            location = "mps"
 """
    diff_file = path.temp()/"pytorch_mps.diff"
    with open(str(diff_file),"w") as f:
      f.write(DIFF)

    #################

    PYTHON = dep.instance("python")
    serpy_file = PYTHON.site_packages_dir/"torch"/"serialization.py"

    self._builder._installcommands += [
      command.Command(
        [ "pip3","install",
          "--pre","torch",
          "torchvision","torchaudio",
          "--extra-index-url","https://download.pytorch.org/whl/nightly/cpu"
        ]
      ),
      # command to patch the file
      command.Command(
        [ "patch", "-p0", "-i", str(diff_file), str(serpy_file) ]
      ),
    ]
  @property
  def _fetcher(self):
    return dep.NopFetcher(name="pytorch_for_mps")

###############################################################################
class pytorch(dep.switch(linux=_pytorch_from_source, \
                         macos=_pytorch_for_mps)):
  def __init__(self):
    super().__init__()
  def env_init(self):
    log.marker("registering pytorch SDK(%s)"%self.VERSION)



import os
from obt.deco import Deco

global deco

deco = Deco()

def output(x):
    if not "OBT_QUIET" in os.environ:
      print(x)

def rgb(r,g,b,string):
    if not "OBT_QUIET" in os.environ:
      print(deco.rgbstr(r,g,b,string))

def marker(string):
  output(deco.bright(string))



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, string
import obt.path
import obt.dep
from obt.path import *
import obt.deco
deco = obt.deco.Deco()

#################################################################################

def find(word):
 def _find(path):
  if os.path.exists(str(path)):
    with open(path, "rb") as fp:
     for n, line in enumerate(fp):
      try:
        line_as_str = line.decode("utf-8")
        if word in line_as_str:
          yield n+1, line_as_str
      except Exception:
        pass
 return _find

#################################################################################

class result:
 def __init__(self,path,lineno,text):
  self.path = path
  self.lineno = lineno
  self.text = text

#################################################################################

ext_set = set(os.environ["OBT_SEARCH_EXTLIST"].split(":"))

#################################################################################

ignore_folder_keys = set(["/obj/", "/pluginobj/","/.build/"])

#################################################################################

def search_at_root(word, root,ignore_set = ignore_folder_keys):
 finder = find(word)
 results = list()
 for root, dirs, files in os.walk(root):
  for f in files:
   path = os.path.join(root, f)
   spl = os.path.splitext(path)
   ext = spl[1]
   ignore = False
   for item in ignore_set:
       if (spl[0].find(item)!=-1):
           ignore = True
   if not ignore:
    if (finder!=None) and ext in ext_set:
     for line_number, line in finder(path):
      line = line.replace("\n","")
      res = result(path,line_number,line)
      results.append(res)
 return results

#################################################################################

p = os.environ["OBT_ROOT"]
pthspec = p.split(":")

#################################################################################

if "OBT_SEARCH_PATH" in os.environ:
  p = os.environ["OBT_SEARCH_PATH"]
  pthspec += p.split(":")

#################################################################################

#print(pthspec)
default_pathlist = []
for p in pthspec:
  default_pathlist += [Path(p)]

#################################################################################

def execute(word,path_list = default_pathlist):
  for path in path_list:
   results = search_at_root(word,str(path))
   have_results = len(results)!=0
   if have_results:
    print("/////////////////////////////////////////////////////////////")
    print("// path : %s" % path)
    print("/////////")
    root = str(obt.path.project_list())+"/"
    for item in results:
      pathstr = str(item.path)
      pathstr = pathstr.replace(str(root),"")
      deco_path = "%-*s"%(72,deco.path(pathstr))
      deco_lino = "%s %s"%(deco.bright("Line"),deco.yellow(item.lineno))
      deco_lino = "%-*s"%(37,deco_lino)
      deco_text = deco.val(item.text.strip())
      print("%s%s %s" % (deco_path, deco_lino, deco_text))

#################################################################################

def execute_at(word_list,path_list,remove_root=None):
  for path in path_list:
   print("/////////////////////////////////////////////////////////////")
   print("// searching path : %s" % path)
   print("/////////")
   for word in word_list:
     results = search_at_root(word,str(path))
     have_results = len(results)!=0
     if have_results:
       for item in results:
         pathstr = str(item.path)
         if remove_root!=None:
           r_root = str(remove_root)+"/"
           pathstr = pathstr.replace(str(r_root),"")
         deco_path = "%-*s"%(72,deco.path(pathstr))
         deco_lino = "%s %s"%(deco.magenta("Line"),deco.yellow(item.lineno))
         deco_lino = "%-*s"%(37,deco_lino)
         deco_text = deco.val(item.text.strip())
         print("%s%s %s" % (deco_path, deco_lino, deco_text))


#################################################################################

def visit(word,visitor, path_list = default_pathlist):
  for path in path_list:
   results = search_at_root(word,str(path))
   have_results = len(results)!=0
   print(have_results)
   if have_results:
     visitor.onPath(path)
     for item in results:
       visitor.onItem(item)



#!/usr/bin/env python3

import os, sys, subprocess, argparse
from obt import command, path, pathtools
from obt import sdk, dep, subspace, conan, cmake_gen

this_dir = path.directoryOfInvokingModule()

argparser = argparse.ArgumentParser(description="Build and install the ios_metal_app")
argparser.add_argument("--clean", action="store_true", help="clean build")
argparser.add_argument("--install", action="store_true", help="install the app on the connected device")
args = argparser.parse_args()

do_install = args.install
do_clean = args.clean

##############################################

IOS_SUBSPACE_DIR = subspace.descriptor("ios")._subsrc
IOS_SDK = sdk.descriptor("aarch64","ios")
prefix = path.subspace_dir()
SRC_DIR = this_dir/".."

my_build_dir = prefix/"builds"/"metalapp"
pathtools.ensureDirectoryExists(my_build_dir)

assert(subspace.current() == "ios")

##############################################

IOS_SDK = sdk.descriptor("aarch64","ios")
lexertl = dep.require("lexertl14")
parsertl = dep.require("parsertl14")
the_environ = conan.environment()
the_environ.update(IOS_SDK._environment)
the_environ.update({
  "OBT_SUBSPACE_BUILD_DIR": prefix/"builds",
  "OBT_SUBSPACE_LIB_DIR": prefix/"lib",
  "OBT_SUBSPACE_BIN_DIR": prefix/"bin",
})

##############################################

os.chdir(my_build_dir)

##############################################

if not (my_build_dir/".build-metal").exists():
  do_clean = True

##############################################

command.run(["cp", IOS_SUBSPACE_DIR/"CMakeListsMetal.txt", my_build_dir/"CMakeLists.txt"], do_log=True)
command.run(["cp", IOS_SUBSPACE_DIR/"InfoMetal.plist", my_build_dir/"Info.plist"], do_log=True)

##############################################
# generate cmake project
##############################################

do_cmakegen = False

if do_cmakegen:
  WS = cmake_gen.workspace("ObtMetalApp")

  WS.findLibrary("UIKit")
  WS.findLibrary("Foundation")
  WS.findLibrary("CoreGraphics")
  WS.findLibrary("Metal")
  WS.findLibrary("QuartzCore")
  WS.findPackage("glm",CONFIG=True)
  WS.findPackage("ZLIB")
  WS.findPackage("Boost",COMPONENTS=["system","filesystem"])

  WS.setVar("DLL_NAME", "${PROJECT_NAME}_DLL")
  WS.setVar("APP_NAME", "${PROJECT_NAME}_APP")

  APP = WS.createExecutable("${APP_NAME}")
  DLL = WS.createSharedLibrary("${DLL_NAME}")
  APP.dependsOn(DLL)
  APP.add_src("${SRC_DIR}/main_metal.mm")
  DLL.add_src("${SRC_DIR}/dll.mm")


  EMITTER = cmake_gen.emitter("ios")
  WS.emit(EMITTER)
  assert(False)

##############################################
if do_clean: # CLEAN BUILD
##############################################
  conan.require(prefix,my_build_dir,[
    "zlib/1.2.11",
    "boost/1.84.0",
    "glm/cci.20230113",
    "zeromq/4.3.4",
    "zmqpp/4.2.0",
    #"openimageio/2.5.9.0",
    "stb/cci.20230920",
    "lexertl14/tweakoz-obt@user/channel",
  ])
  pathtools.mkdir(my_build_dir/".build-metal",clean=True)
  os.chdir(my_build_dir/".build-metal")
  command.run(["cmake", "..", "-G", "Xcode","-DSRC_DIR=%s"%str(SRC_DIR)], environment=the_environ, do_log=True)
  command.run(["cmake", "--build", ".","--config", "Release"], environment=the_environ, do_log=True)

##############################################
else: # INCREMENTAL BUILD
##############################################
  os.chdir(my_build_dir/".build-metal")
  command.run(["cmake", "--build", ".","--config", "Release"], environment=the_environ, do_log=True)

##############################################

app_bundle_dir = my_build_dir / ".build-metal/Release-iphoneos/ios_metal_app.app"
# Verify the executable exists within the bundle
executable_path = app_bundle_dir / "ios_metal_app"
if not os.path.exists(executable_path):
    print(f"Executable not found at {executable_path}")
    sys.exit(1)

##############################################
# Install the app on the connected device
##############################################

if do_install:
  IOS_SDK.install_app("com.example.metalapp", app_bundle_dir)



#!/usr/bin/env python3

################################################################################

from obt import path, wget, crypt 
import sys, os, random, numpy, argparse
import base64

parser = argparse.ArgumentParser(description='assetpak generator')
parser.add_argument("-o", '--output', type=str, help='output file', required=False)
parser.add_argument("-i", '--input', type=str, help='input file', required=False)
parser.add_argument("-s", '--string', type=str, help='string', required=False)
parser.add_argument("-p", '--passphrase', type=str, help='passphrase', required=True)
args = vars(parser.parse_args())

output = args['output']
input = args.get('input')
input_string = args.get('string')
passphrase = args['passphrase']

if input_string:
  #strip quotes surrounding the string if they exist
  if input_string[0] == '"' and input_string[-1] == '"':
    input_string = input_string[1:-1]

  # Encrypt the string
  encrypted_data = crypt.encrypt_string(input_string, passphrase)
  # Encode the encrypted data to an alphanumeric string using base64
  encoded_string = base64.urlsafe_b64encode(encrypted_data).decode('utf-8')
  print(encoded_string)
  # Write the encoded string to the output file
else:
  crypt.encrypt_file(input, output, passphrase)



###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, inspect, tarfile, shutil,sys
from pathlib import Path
from yarl import URL
import importlib.util
import obt.path, obt.host
from obt import patch
from obt.command import Command, run
from obt.deco import Deco
from obt.wget import wget
from obt import pathtools, cmake, make, path, git, host, _globals, buildtrace
from obt._dep_dl import downloadAndExtract
from obt._dep_impl import require 
from obt import pathtools 

deco = Deco()

###############################################################################

class Fetcher:
  def __init__(self,name):
    self._name = name
    self._debug = False
    self._patchdict = dict()
  def patch(self):
    for filepath in self._patchdict:
      item_dict = self._patchdict[filepath]
      patch.patch_with_dict(filepath,item_dict)

###############################################################################

class DepotToolsFetcher(Fetcher):
  def __init__(self,name):
    super().__init__(name)
    self._fetch_id = ""
    self._enable_hooks = False
  ###########################################
  def addSubFetcher(self,name):
    pass
  ###########################################
  def fetch(self,dest):
    p = path.builds()/"depot_tools"
    cmdlist = [
      "fetch",
      "--nohooks",
      "--no-history",
      self._fetch_id      
    ]
    env = {
      "PATH": str(p)+":"+os.environ["PATH"]
    }
    wd = path.builds()/self._fetch_id
    pathtools.ensureDirectoryExists(wd)
    rval = run(cmdlist,environment=env, working_dir=wd,do_log=True)
    return rval==0

###############################################################################

class CompositeFetcher(Fetcher):
  def __init__(self,name):
    super().__init__(name)
    self._subfetchers = []
  ###########################################
  def addSubFetcher(self,fetcher):
    self._subfetchers += [fetcher]
  ###########################################
  def fetch(self,dest):
    for item in self._subfetchers:
      rval = item.fetch(dest)
      if rval==False:
        print("subfetcher<%s> failed!" % item)
        return False
    return True
###############################################################################

class GitFetcher(Fetcher):
  ###########################################
  def __init__(self,name):
    super().__init__(name)
    self._git_url = ""
    self._revision = ""
    self._recursive = False
    self._cache = True
  ###########################################
  def descriptor(self):
    return "%s (git-%s)" % (self._name,self._revision)
  ###########################################
  def fetch(self,dest):
    with buildtrace.NestedBuildTrace({ 
      "op": "fetch(git)",
      "url": self._git_url,
      "rev": self._revision,
      "dest": dest, 
      "recursive": self._recursive, 
      "cache": self._cache,
      "shallow": False }) as nested:
        return git.Clone(self._git_url,
                         dest,
                         rev=self._revision,
                         recursive=self._recursive,
                         cache=self._cache,
                         shallow=False)
###############################################################################

class GithubFetcher(Fetcher): # github specific git fetcher
  ###########################################
  def __init__(self,
               name=None,
               repospec=None,
               revision="master",
               recursive=False,
               cache=False,
               md5val=None,
               shallow=True,
               disable_tarball=False,
               patchdict=dict(),
               dest_dir_override = None):
    super().__init__(name)
    # todo : allow user control over protocols
    #  since ssh requires key setup..
    self._git_url = "git@github.com:"+repospec
    self._repospec = repospec
    #self._git_url = "http://github.com/"+repospec
    self._revision = revision
    self._recursive = recursive
    self._cache = cache
    self._shallow = shallow
    self._md5val = md5val
    self._force_clone = False
    self._disable_tarball = not (self._shallow and (not self._recursive) and (not self._force_clone))
    if disable_tarball:
      self._disable_tarball = True
    self._patchdict = patchdict
    self._dest_dir_override = dest_dir_override
    #print(self._git_url)
  ###########################################
  def descriptor(self):
    return "%s (git-%s)" % (self._name,self._revision)
  ###########################################
  def update(self,dest):
    print(self._disable_tarball)
    if self._disable_tarball:
      return git.checkout_update(dest,self._revision)
    else:
      return False
  ###########################################
  def fetch(self,dest):
    ####################################################
    # try to use githubs tarball fetch feature
    # when appropriate because its hella faster
    ####################################################
    #print("UseTarball<%s>"%use_tarball)
    #print(_globals.getOptions())

    if self._dest_dir_override!=None:
      dest = self._dest_dir_override

    use_tarball = not self._disable_tarball

    with buildtrace.NestedBuildTrace({ "op": "fetch(github)", "repospec": self._repospec, "use_tarball": use_tarball, "recursive": self._recursive, "revision": self._revision }) as nested:

      if "usegitclone" in _globals.getOptions():
       if _globals.getOptions()["usegitclone"]==True:
         use_tarball = False
   
      if self._debug:
        print(deco.bright("GithubFetcher<%s> use_tarball<%s>"%(self._name,use_tarball)))

      ####################################################
      if use_tarball:
      ####################################################
        retc = git.fetch_tarball_from_github(
           repospec=self._repospec,
           revision=self._revision,
           md5val=self._md5val,
           destdir=dest)
        return retc==0
      ####################################################
      else: # tried and true way
      ####################################################
        return git.Clone(self._git_url,
                         dest,
                         rev=self._revision,
                         recursive=self._recursive,
                         cache=self._cache,
                         shallow=self._shallow)

###############################################################################

class SvnFetcher(Fetcher):
  ###########################################
  def __init__(self,name):
    super().__init__(name)
    self._url = ""
    self._revision = ""
    self._recursive = False
    self._cache = True
  ###########################################
  def descriptor(self):
    return "%s (svn-%s)" % (self._name,self._revision)
  ###########################################
  def fetch(self,dest):
    url = self._url+"/"+self._revision
    cmd = ["svn","checkout", url, dest]
    retc = run(cmd)
    return retc==0
  ###########################################

###############################################################################

class WgetFetcher(Fetcher):
  ###########################################
  def __init__(self,name):
    super().__init__(name)
    self._fname = ""
    self._url = ""
    self._md5 = ""
    self._arcpath = ""
    self._arctype = ""
    self._dest = path.builds()/self._name
    self._arc_options = []
  ###########################################
  def descriptor(self):
    return "%s (wget: %s)" % (self._name,self._url)
  ###########################################
  def fetch(self,dest):
    from yarl import URL
    url = URL(self._url)
    with buildtrace.NestedBuildTrace({ "op": "fetch(wget)", "url": url, "md5": self._md5, "dest": self._dest, "fname": self._fname, "arctype": self._arctype }) as nested:
      self._arcpath = downloadAndExtract([url],
                                         self._fname,
                                         self._arctype,
                                         self._md5,
                                         self._dest,
                                         arc_options=self._arc_options)
      return self._arcpath!=None

  ###########################################

###############################################################################

class NopFetcher(Fetcher):
  ###########################################
  def __init__(self,name):
    super().__init__(name)
    self._revision = ""
  ###########################################
  def descriptor(self):
    return "%s (%s)" % (self._name,self._revision)
  ###########################################
  def update(self,source_root):
    print("NOPFETCHER not updating for <%s> at src_root<%s>" % (self._name, source_root))
    return True
  ###########################################
  def fetch(self,dest):
    return True
  ###########################################



from setuptools import setup, find_packages, Command
from setuptools.command.install import install
import os
import stat

version = "0.0.258"

# Read the long description from the README file
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

def package_files(directory):
    data_files = []
    for root, dirs, files in os.walk(directory):
        for filename in files:
            if not (filename.endswith('.pyc') or '.egg-info' in root or '__pycache__' in root):
                filepath = os.path.join(root, filename)
                dest_dir = os.path.join('obt', root)
                data_files.append((dest_dir, [filepath]))
    return data_files

#########################################
# these are tricky
#  bin_priv has to be data so the files can have the x bit set
#  bin_pub goes in scripts (since they are in the venv's path)
#########################################

module_files = package_files('modules') 
example_files = package_files('examples')
test_files = package_files('tests')
binpub_files = [f[1][0] for f in package_files("bin_pub")]
binpriv_files = package_files('bin_priv')
data_files = module_files + example_files + test_files + binpriv_files

#########################################

setup(
    name="ork.build",
    version=version,
    author="Michael T. Mayers",
    author_email="michael@tweakoz.com",
    description="Orkid Build Tools",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/tweakoz/ork.build",
    packages=find_packages(where="scripts"),
    package_dir={"": "scripts"},
    include_package_data=True,
    data_files=data_files,
    scripts=binpub_files,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.10',
    install_requires=[
        'yarl',
        'toposort',
        'os-release',
        'distro',
        "twine",
        "build",
        "conan",
        "pip>=24.1",
	"GitPython"
	# "mdstat", linux only
	#"zfslib", linux only
	#"pyudev", linux only
    ]
)



from obt import subspace, path

###########################################################

def exists(name):
  
  if subspace.is_host_subspace():
    manifest = path.subspace_dir()/"manifest"
    if (manifest/name).exists():
      return True
    else:
      manifest = path.stage()/"manifest"
      return (manifest/name).exists()
  else:
    manifest = path.subspace_dir()/"manifest"
    return (manifest/name).exists()